def migrate(conn, tables):
    updateTablesPriority = ["ProductionOrders", "PicklistOrders"]

    for table in updateTablesPriority:
        if len(tables[table]["fields_Common"]) == 0:
            continue

        strCommonFields = ", ".join(tables[table]["fields_Common"])
        stmt = "INSERT OR REPLACE INTO {} ({}) SELECT {} FROM old.{};"
        conn.query(stmt.format(table, strCommonFields, strCommonFields, table))



from sqlalchemy.testing.requirements import Requirements

from alembic import util
from alembic.util import sqla_compat
from ..testing import exclusions


class SuiteRequirements(Requirements):
    @property
    def schemas(self):
        """Target database must support external schemas, and have one
        named 'test_schema'."""

        return exclusions.open()

    @property
    def autocommit_isolation(self):
        """target database should support 'AUTOCOMMIT' isolation level"""

        return exclusions.closed()

    @property
    def materialized_views(self):
        """needed for sqlalchemy compat"""
        return exclusions.closed()

    @property
    def unique_constraint_reflection(self):
        def doesnt_have_check_uq_constraints(config):
            from sqlalchemy import inspect

            insp = inspect(config.db)
            try:
                insp.get_unique_constraints("x")
            except NotImplementedError:
                return True
            except TypeError:
                return True
            except Exception:
                pass
            return False

        return exclusions.skip_if(doesnt_have_check_uq_constraints)

    @property
    def sequences(self):
        """Target database must support SEQUENCEs."""

        return exclusions.only_if(
            [lambda config: config.db.dialect.supports_sequences],
            "no sequence support",
        )

    @property
    def foreign_key_match(self):
        return exclusions.open()

    @property
    def foreign_key_constraint_reflection(self):
        return exclusions.open()

    @property
    def check_constraints_w_enforcement(self):
        """Target database must support check constraints
        and also enforce them."""

        return exclusions.open()

    @property
    def reflects_pk_names(self):
        return exclusions.closed()

    @property
    def reflects_fk_options(self):
        return exclusions.closed()

    @property
    def sqlalchemy_14(self):
        return exclusions.skip_if(
            lambda config: not util.sqla_14,
            "SQLAlchemy 1.4 or greater required",
        )

    @property
    def sqlalchemy_1x(self):
        return exclusions.skip_if(
            lambda config: util.sqla_2,
            "SQLAlchemy 1.x test",
        )

    @property
    def sqlalchemy_2(self):
        return exclusions.skip_if(
            lambda config: not util.sqla_2,
            "SQLAlchemy 2.x test",
        )

    @property
    def asyncio(self):
        def go(config):
            try:
                import greenlet  # noqa: F401
            except ImportError:
                return False
            else:
                return True

        return self.sqlalchemy_14 + exclusions.only_if(go)

    @property
    def comments(self):
        return exclusions.only_if(
            lambda config: config.db.dialect.supports_comments
        )

    @property
    def alter_column(self):
        return exclusions.open()

    @property
    def computed_columns(self):
        return exclusions.closed()

    @property
    def computed_columns_api(self):
        return exclusions.only_if(
            exclusions.BooleanPredicate(sqla_compat.has_computed)
        )

    @property
    def computed_reflects_normally(self):
        return exclusions.only_if(
            exclusions.BooleanPredicate(sqla_compat.has_computed_reflection)
        )

    @property
    def computed_reflects_as_server_default(self):
        return exclusions.closed()

    @property
    def computed_doesnt_reflect_as_server_default(self):
        return exclusions.closed()

    @property
    def autoincrement_on_composite_pk(self):
        return exclusions.closed()

    @property
    def fk_ondelete_is_reflected(self):
        return exclusions.closed()

    @property
    def fk_onupdate_is_reflected(self):
        return exclusions.closed()

    @property
    def fk_onupdate(self):
        return exclusions.open()

    @property
    def fk_ondelete_restrict(self):
        return exclusions.open()

    @property
    def fk_onupdate_restrict(self):
        return exclusions.open()

    @property
    def fk_ondelete_noaction(self):
        return exclusions.open()

    @property
    def fk_initially(self):
        return exclusions.closed()

    @property
    def fk_deferrable(self):
        return exclusions.closed()

    @property
    def fk_deferrable_is_reflected(self):
        return exclusions.closed()

    @property
    def fk_names(self):
        return exclusions.open()

    @property
    def integer_subtype_comparisons(self):
        return exclusions.open()

    @property
    def no_name_normalize(self):
        return exclusions.skip_if(
            lambda config: config.db.dialect.requires_name_normalize
        )

    @property
    def identity_columns(self):
        return exclusions.closed()

    @property
    def identity_columns_alter(self):
        return exclusions.closed()

    @property
    def identity_columns_api(self):
        return exclusions.only_if(
            exclusions.BooleanPredicate(sqla_compat.has_identity)
        )



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gzip
import logging
import zlib
from io import BytesIO
from os import environ
from typing import Dict, Optional
from time import sleep

import requests

from opentelemetry.exporter.otlp.proto.common._internal import (
    _create_exp_backoff_generator,
)
from opentelemetry.exporter.otlp.proto.common.trace_encoder import (
    encode_spans,
)
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPORTER_OTLP_TRACES_CERTIFICATE,
    OTEL_EXPORTER_OTLP_TRACES_COMPRESSION,
    OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
    OTEL_EXPORTER_OTLP_TRACES_HEADERS,
    OTEL_EXPORTER_OTLP_TRACES_TIMEOUT,
    OTEL_EXPORTER_OTLP_CERTIFICATE,
    OTEL_EXPORTER_OTLP_COMPRESSION,
    OTEL_EXPORTER_OTLP_ENDPOINT,
    OTEL_EXPORTER_OTLP_HEADERS,
    OTEL_EXPORTER_OTLP_TIMEOUT,
)
from opentelemetry.sdk.trace.export import SpanExporter, SpanExportResult
from opentelemetry.exporter.otlp.proto.http import (
    _OTLP_HTTP_HEADERS,
    Compression,
)
from opentelemetry.util.re import parse_env_headers


_logger = logging.getLogger(__name__)


DEFAULT_COMPRESSION = Compression.NoCompression
DEFAULT_ENDPOINT = "http://localhost:4318/"
DEFAULT_TRACES_EXPORT_PATH = "v1/traces"
DEFAULT_TIMEOUT = 10  # in seconds


class OTLPSpanExporter(SpanExporter):

    _MAX_RETRY_TIMEOUT = 64

    def __init__(
        self,
        endpoint: Optional[str] = None,
        certificate_file: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
        timeout: Optional[int] = None,
        compression: Optional[Compression] = None,
        session: Optional[requests.Session] = None,
    ):
        self._endpoint = endpoint or environ.get(
            OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
            _append_trace_path(
                environ.get(OTEL_EXPORTER_OTLP_ENDPOINT, DEFAULT_ENDPOINT)
            ),
        )
        self._certificate_file = certificate_file or environ.get(
            OTEL_EXPORTER_OTLP_TRACES_CERTIFICATE,
            environ.get(OTEL_EXPORTER_OTLP_CERTIFICATE, True),
        )
        headers_string = environ.get(
            OTEL_EXPORTER_OTLP_TRACES_HEADERS,
            environ.get(OTEL_EXPORTER_OTLP_HEADERS, ""),
        )
        self._headers = headers or parse_env_headers(headers_string)
        self._timeout = timeout or int(
            environ.get(
                OTEL_EXPORTER_OTLP_TRACES_TIMEOUT,
                environ.get(OTEL_EXPORTER_OTLP_TIMEOUT, DEFAULT_TIMEOUT),
            )
        )
        self._compression = compression or _compression_from_env()
        self._session = session or requests.Session()
        self._session.headers.update(self._headers)
        self._session.headers.update(_OTLP_HTTP_HEADERS)
        if self._compression is not Compression.NoCompression:
            self._session.headers.update(
                {"Content-Encoding": self._compression.value}
            )
        self._shutdown = False

    def _export(self, serialized_data: str):
        data = serialized_data
        if self._compression == Compression.Gzip:
            gzip_data = BytesIO()
            with gzip.GzipFile(fileobj=gzip_data, mode="w") as gzip_stream:
                gzip_stream.write(serialized_data)
            data = gzip_data.getvalue()
        elif self._compression == Compression.Deflate:
            data = zlib.compress(bytes(serialized_data))

        return self._session.post(
            url=self._endpoint,
            data=data,
            verify=self._certificate_file,
            timeout=self._timeout,
        )

    @staticmethod
    def _retryable(resp: requests.Response) -> bool:
        if resp.status_code == 408:
            return True
        if resp.status_code >= 500 and resp.status_code <= 599:
            return True
        return False

    def export(self, spans) -> SpanExportResult:
        # After the call to Shutdown subsequent calls to Export are
        # not allowed and should return a Failure result.
        if self._shutdown:
            _logger.warning("Exporter already shutdown, ignoring batch")
            return SpanExportResult.FAILURE

        serialized_data = encode_spans(spans).SerializeToString()

        for delay in _create_exp_backoff_generator(
            max_value=self._MAX_RETRY_TIMEOUT
        ):

            if delay == self._MAX_RETRY_TIMEOUT:
                return SpanExportResult.FAILURE

            resp = self._export(serialized_data)
            # pylint: disable=no-else-return
            if resp.status_code in (200, 202):
                return SpanExportResult.SUCCESS
            elif self._retryable(resp):
                _logger.warning(
                    "Transient error %s encountered while exporting span batch, retrying in %ss.",
                    resp.reason,
                    delay,
                )
                sleep(delay)
                continue
            else:
                _logger.error(
                    "Failed to export batch code: %s, reason: %s",
                    resp.status_code,
                    resp.text,
                )
                return SpanExportResult.FAILURE
        return SpanExportResult.FAILURE

    def shutdown(self):
        if self._shutdown:
            _logger.warning("Exporter already shutdown, ignoring call")
            return
        self._session.close()
        self._shutdown = True

    def force_flush(self, timeout_millis: int = 30000) -> bool:
        """Nothing is buffered in this exporter, so this method does nothing."""
        return True


def _compression_from_env() -> Compression:
    compression = (
        environ.get(
            OTEL_EXPORTER_OTLP_TRACES_COMPRESSION,
            environ.get(OTEL_EXPORTER_OTLP_COMPRESSION, "none"),
        )
        .lower()
        .strip()
    )
    return Compression(compression)


def _append_trace_path(endpoint: str) -> str:
    if endpoint.endswith("/"):
        return endpoint + DEFAULT_TRACES_EXPORT_PATH
    return endpoint + f"/{DEFAULT_TRACES_EXPORT_PATH}"



from typing import (
    Any,
    Union,
    Optional,
)

from modelcommon.model import SitefResponse
from model import (
    CancelPaymentOrderData,
    ConfirmFinishPaymentData,
    TestConnectionData,
    StartPaymentData,
    FinishPaymentData,
)
from paymentprocessor import PaymentProcessor
from ._SitefPaymentProcessor import SitefPaymentProcessor


class SitefGiftCardPaymentProcessor(PaymentProcessor):

    def __init__(
        self,               # type: SitefGiftCardPaymentProcessor
        message_bus,        # type: MbContextMessageBus
        dialog_manager,     # type: DialogManager
        config,             # type: Group
        logger,             # type: Logger
        cache,              # type: Cache
    ):
        # type: (...) -> None

        super(SitefGiftCardPaymentProcessor, self).__init__(message_bus, dialog_manager, config, logger, cache)

        self.sitef_processor = SitefPaymentProcessor(
            message_bus=message_bus,
            dialog_manager=dialog_manager,
            config=config,
            logger=logger,
            cache=cache,
        )

        self.allowed_payments = config.find_values("AllowedPayments") or []
        self.enabled_additional_transactions = config.find_values("EnabledAdditionalTransactions") or []

        key_cancel_additional_transactions = "CancelEnabledAdditionalTransactions"
        self.cancel_enabled_additional_transactions = config.find_values(key_cancel_additional_transactions) or []

        default_disabled_payments = ["10", "17", "18", "19", "27", "28", "29", "34", "35", "36", "42", "3988"]
        self.disabled_payments = config.find_values("DisabledPayments") or default_disabled_payments

        self.additional_data = self._get_additional_data()
        self.cancel_additional_data = self._get_cancel_additional_data()

    def connection_status(
        self,                   # type: SitefGiftCardPaymentProcessor
        test_connection_data,   # type: TestConnectionData
    ):
        # type: (...) -> bool

        return self.sitef_processor.connection_status(test_connection_data=test_connection_data)

    def add_payment(
        self,                   # type: SitefGiftCardPaymentProcessor
        start_payment_data,     # type: StartPaymentData
    ):
        # type: (...) -> Any[Union[SitefResponse, str]]

        return self.sitef_processor.add_payment(
            start_payment_data=start_payment_data,
            additional_params=self.additional_data,
        )

    def cancel_payment_order(
        self,                           # type: SitefGiftCardPaymentProcessor
        cancel_payment_order_data,      # type: CancelPaymentOrderData
        additional_params="",           # type: Optional[str]
    ):
        # type: (...) -> Any[Union[SitefResponse, str]]

        return self.sitef_processor.cancel_payment_order(
            cancel_payment_order_data=cancel_payment_order_data,
            additional_params=self.cancel_additional_data,
        )

    def cancel_payment(
        self,  # type: SitefGiftCardPaymentProcessor
    ):
        # type: (...) -> str

        return self.sitef_processor.cancel_payment()

    def finish_payment(
        self,                   # type: SitefGiftCardPaymentProcessor
        finish_payment_data,    # type: FinishPaymentData
    ):
        # type: (...) -> bool

        return self.sitef_processor.finish_payment(finish_payment_data=finish_payment_data)

    def confirm_finish_payment(
        self,                           # type: SitefGiftCardPaymentProcessor
        confirm_finish_payment_data,    # type: ConfirmFinishPaymentData
    ):
        # type: (...) -> None

        pass

    def get_name(
        self,   # type: SitefGiftCardPaymentProcessor
    ):
        # type: (...) -> str

        return self.sitef_processor.get_name()

    def get_document(
        self,       # type: SitefGiftCardPaymentProcessor
        pos_id,     # type: str
    ):
        # type: (...) -> str

        return self.sitef_processor.get_document(pos_id=pos_id)

    def stop(
        self,   # type: SitefGiftCardPaymentProcessor
    ):
        # type: (...) -> None

        self.sitef_processor.stop()

    def get_last_receipt(
        self,           # type: SitefGiftCardPaymentProcessor
        eft_processor,  # type: str
    ):
        # type: (...) -> Optional[str]

        return self.sitef_processor.get_last_receipt(eft_processor=eft_processor)

    def _get_additional_data(
        self,   # type: SitefGiftCardPaymentProcessor
    ):
        # type: (...) -> str

        additional_data_list = []

        if self.disabled_payments:
            additional_data_list.append("[{}]".format(';'.join(self.disabled_payments)))

        if self.allowed_payments:
            additional_data_list.append("{{TransacoesHabilitadas={}}}".format(';'.join(self.allowed_payments)))

        if self.enabled_additional_transactions:
            additional_transactions = ';'.join(self.enabled_additional_transactions)
            additional_data_list.append("{{TransacoesAdicionaisHabilitadas={}}}".format(additional_transactions))

        return ';'.join(additional_data_list)

    def _get_cancel_additional_data(
        self,   # type: SitefGiftCardPaymentProcessor
    ):
        # type: (...) -> str

        cancel_additional_data_list = []

        if self.cancel_enabled_additional_transactions:
            additional_transactions = ';'.join(self.cancel_enabled_additional_transactions)
            cancel_additional_data_list.append("{{TransacoesAdicionaisHabilitadas={}}}".format(additional_transactions))

        return ';'.join(cancel_additional_data_list)
